{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Import libraries"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "import ee\n",
    "import geemap"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Create an interactive map"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "Map = geemap.Map(center=[40, -100], zoom=4)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Add Earth Engine Python script"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "# Add Earth Engine dataset\n",
    "image = ee.Image(\"USGS/SRTMGL1_003\")\n",
    "\n",
    "# Import roads.\n",
    "grip4_africa = ee.FeatureCollection(\n",
    "  'projects/sat-io/open-datasets/GRIP4/Africa')\n",
    "grip4_europe = ee.FeatureCollection(\n",
    "  'projects/sat-io/open-datasets/GRIP4/Europe')\n",
    "grip4_north_america = ee.FeatureCollection(\n",
    "  'projects/sat-io/open-datasets/GRIP4/North-America')\n",
    "\n",
    "# Check sizes\n",
    "print('Grip4 Africa size', grip4_africa.size())\n",
    "print('Grip4 North America size', grip4_north_america.size())\n",
    "print('Grip4 Europe size', grip4_europe.size())\n",
    "\n",
    "Map.setCenter(3.6, 32.5, 11)\n",
    "\n",
    "# Display roads\n",
    "Map.addLayer(grip4_africa.style(**{'color': '413B3A', 'width':1}),\n",
    "             {},\n",
    "             'Grip4 Africa')\n",
    "Map.addLayer(grip4_north_america.style(**{'color': '413B3A', 'width':1}),\n",
    "             {},\n",
    "             'Grip4 North America')\n",
    "Map.addLayer(grip4_europe.style(**{'color': '413B3A', 'width':1}),\n",
    "             {},\n",
    "             'Grip4 Europe')\n",
    "\n",
    "# Import simplified countries\n",
    "countries = ee.FeatureCollection('USDOS/LSIB_SIMPLE/2017')\n",
    "\n",
    "# Function to calculate feature's geometry area and adds it as a property\n",
    "def addArea(feature):\n",
    "  return feature.set({\n",
    "    'areaKm': feature.geometry().area().divide(1000 * 1000) # km2 squared\n",
    "  })\n",
    "\n",
    "\n",
    "# Map the area getting function over the FeatureCollection.\n",
    "countriesArea = countries.map(addArea)\n",
    "\n",
    "# Filter to largest country in Africa\n",
    "Algeria = countriesArea.filter(ee.Filter.inList('country_na', ['Algeria']))\n",
    "\n",
    "# Display selected countries\n",
    "Map.addLayer(Algeria.style(**{'fillColor': 'b5ffb4', 'color': '00909F', 'width': 1.0}),\n",
    "             {},\n",
    "             'Algeria')\n",
    "\n",
    "# Calculate road length per country for the associated GRIP dataset.\n",
    "def roadLength4Country(country, grip4):\n",
    "\n",
    "  # Join roads to countries\n",
    "  interesectsFilter = ee.Filter.intersects({\n",
    "    'leftField': '.geo',\n",
    "    'rightField': '.geo',\n",
    "    'maxError': 10\n",
    "  })\n",
    "\n",
    "  grip4Selected = grip4.filterBounds(country)\n",
    "\n",
    "  countriesWithRds = ee.Join.saveAll('roads').apply({\n",
    "    'primary': country,\n",
    "    'secondary': grip4Selected,\n",
    "    'condition': interesectsFilter\n",
    "  }).filter(ee.Filter.neq('roads', None))\n",
    "\n",
    "  # Return country with road length and roads per km square km set\n",
    "\n",
    "def func_ibq(country):\n",
    "    roadsList = ee.List(country.get('roads'))\n",
    "    roadLengths = roadsList.map(function(road) {\n",
    "      return ee.Feature(road).intersection(country, 10).length(10)\n",
    "    })\n",
    "    roadLength = ee.Number(roadLengths.reduce(ee.Reducer.sum()))\n",
    "    return country.set({\n",
    "      'roadLength': roadLength.divide(1000), # Convert to km\n",
    "      'roadsPerArea': roadLength.divide(ee.Number(country.get('areaKm')))\n",
    "    })\n",
    "\n",
    "  return countriesWithRds.map(func_ibq\n",
    ").select(['country_na', 'areaKm','roadLength', 'roadsPerArea'])\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    ").select(['country_na', 'areaKm','roadLength', 'roadsPerArea'])\n",
    "\n",
    "\n",
    "# Apply road length function to Algeria\n",
    "roadLengthAlgeria = roadLength4Country(Algeria, grip4_africa)\n",
    "\n",
    "# Print statics for Algeria for roads per area\n",
    "print('Roads statistics in Algeria', roadLengthAlgeria)\n",
    "\n",
    "# Function to add line length in km\n",
    "def addLength(feature):\n",
    "  return feature.set({\n",
    "    'lengthKm': feature.length().divide(1000) # km squared\n",
    "  })\n",
    "\n",
    "\n",
    "# Calculate line lengths for all roads in Africa\n",
    "grip4_africaLength = grip4_africa.map(addLength)\n",
    "\n",
    "# Compare with other value\n",
    "print(grip4_africaLength.limit(1))\n",
    "\n",
    "# Repeat analysis to calculate length of all roads\n",
    "# Filter the table geographically: only keep roads in Algeria\n",
    "grip4_Algeria = grip4_africaLength.filterBounds(Algeria)\n",
    "\n",
    "# Visualize the output\n",
    "Map.addLayer(grip4_Algeria.style(**{'color': 'green', 'width': 2.0}),\n",
    "             {},\n",
    "             'Algeria roads')\n",
    "\n",
    "# Sum the lengths for roads in Algeria\n",
    "sumLengthKmAlgeria = ee.Number(\n",
    "  # Reduce to get the sum\n",
    "  grip4_Algeria.reduceColumns(ee.Reducer.sum(), ['lengthKm']) \\\n",
    "    .get('sum')\n",
    ")\n",
    "\n",
    "# Print the result\n",
    "print('Length of all roads in Algeria', sumLengthKmAlgeria)\n",
    "\n",
    "# Repeat analysis again to calculate length of all roads using rasters\n",
    "# Convert to raster\n",
    "empty = ee.Image().float()\n",
    "\n",
    "grip4_africaRaster = empty.paint({\n",
    "  'featureCollection': grip4_africaLength,\n",
    "  'color': 'lengthKm'\n",
    "}).gt(0)\n",
    "\n",
    "Map.addLayer(grip4_africaRaster,\n",
    "             {'palette': ['orange'], 'max': 1},\n",
    "             'Rasterized roads')\n",
    "\n",
    "# Add reducer output to the Features in the collection.\n",
    "AlgeriaRoadLength = ee.Image.pixelArea() \\\n",
    "  .addBands(grip4_africaRaster) \\\n",
    "  .reduceRegions({\n",
    "    'collection': Algeria,\n",
    "    'reducer': ee.Reducer.sum(),\n",
    "    'scale': 100,\n",
    "\n",
    "def func_bks(feature):\n",
    "    num = ee.Number.parse(feature.get('area'))\n",
    "    return feature.set('length', num.divide(1000).sqrt().round())\n",
    "\n",
    "}).map(func_bks)\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "# Print the first feature, to illustrate the result.\n",
    "print('Length of all roads in Algeria calculated via rasters',\n",
    "      ee.Number(AlgeriaRoadLength.first().get('length')))\n",
    "\n",
    "# Calculate line lengths for all roads in North America and Europe\n",
    "grip4_north_americaLength = grip4_north_america.map(addLength)\n",
    "grip4_europeLength = grip4_europe.map(addLength)\n",
    "\n",
    "# Merge vectors\n",
    "roadLengthMerge = grip4_africaLength.merge(grip4_north_americaLength) \\\n",
    "                      .merge(grip4_europeLength)\n",
    "\n",
    "# Convert to raster\n",
    "empty = ee.Image().float()\n",
    "\n",
    "roadLengthMergeRaster = empty.paint({\n",
    "  'featureCollection': roadLengthMerge,\n",
    "  'color': 'roadsPerArea'\n",
    "}).gt(0)\n",
    "\n",
    "# Filter to largest countries in Africa, North America and Europe\n",
    "countriesSelected = countries \\\n",
    "    .filter(ee.Filter.inList('country_na', ['Algeria', 'Canada', 'France']))\n",
    "\n",
    "# Clip image to only countries of analysis\n",
    "roadLengthMergeRasterClipped = roadLengthMergeRaster \\\n",
    "    .clipToCollection(countriesSelected)\n",
    "\n",
    "# Add reducer output to the Features in the collection.\n",
    "countriesRoadLength = ee.Image.pixelArea() \\\n",
    "  .addBands(roadLengthMergeRasterClipped) \\\n",
    "  .reduceRegions({\n",
    "    'collection': countriesSelected,\n",
    "    'reducer': ee.Reducer.sum(),\n",
    "    'scale': 100,\n",
    "\n",
    "def func_fgy(feature):\n",
    "    num = ee.Number.parse(feature.get('area'))\n",
    "    return feature.set('length', num.divide(1000).sqrt().round())\n",
    "\n",
    "}).map(func_fgy)\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "# Compute totaled road lengths in km, grouped by country\n",
    "print('Length of all roads in Canada',\n",
    "    countriesRoadLength.filter(ee.Filter.equals('country_na', 'Canada')) \\\n",
    "        .aggregate_sum('length'))\n",
    "print('Length of all roads in France',\n",
    "    countriesRoadLength.filter(ee.Filter.equals('country_na', 'France')) \\\n",
    "        .aggregate_sum('length'))\n",
    "\n",
    "# Examine effect of scale on raster\n",
    "Map.addLayer(grip4_africaRaster.reproject({'crs': 'EPSG:4326', 'scale': 100}),\n",
    "             {'palette': ['orange'], 'max': 1},\n",
    "             'Rasterized roads 100 m')\n",
    "\n",
    "# LGTM (nclinton). Reformatted. Added a setCenter."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Display the interactive map"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "Map"
   ]
  }
 ],
 "metadata": {
  "anaconda-cloud": {},
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.6.1"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 4
}
